class TrainViewController {
    constructor(view) {
        this.view = view
        this.loader = view.querySelector(".loader-button")
        this.datasetNotFound = view.querySelector(".dataset-not-found")
        this.spinner = view.querySelector(".loader-button .spinner")
        this.trainButton = view.querySelector(".loader-button button")
        this.trainInterval = null

        if(MnistModel.instance.datasetNotFound === true) {
            this.loader.style.display = "none"
            this.datasetNotFound.style.display = "block"
        }
        else {
            if(MnistModel.instance.duringTraining) {
                this.displayTrainingProgress()
                MnistModel.instance.addTrainEventListener(this.onTrainingDone.bind(this))
            }

            this.trainButton.addEventListener("click", this.train.bind(this))
        }
    }

    displayTrainingProgress() {
        if(!this.trainInterval && MnistModel.instance.trainStartTime) {
            this.trainButton.disabled = true
            this.spinner.classList.add("loading")

            const display = () => {
                if(MnistModel.instance.trainStartTime) {
                    const t = Math.round((Date.now() - MnistModel.instance.trainStartTime) / 1000)
                    const s = t % 60
                    const m = (t - s) / 60
                    this.trainButton.innerText = `${m}m ${s}s`
                }
            }

            display()
            this.trainInterval = setInterval(display, 1000)
        }
    }

    onTrainingDone(error) {
        if(this.trainInterval) {
            clearInterval(this.trainInterval)
            this.trainInterval = null
        }
        this.spinner.classList.remove("loading")
        this.trainButton.disabled = false
        this.trainButton.innerText = error ? "Error" : "Done"
    }

    train() {
        if(MnistModel.instance.duringTraining) {
            return
        }

        MnistModel.instance.train()
        MnistModel.instance.addTrainEventListener(this.onTrainingDone.bind(this))
        this.displayTrainingProgress()
    }

    dispose() {
        if(this.trainInterval) {
            clearInterval(this.trainInterval)
            this.trainInterval = null
        }
        MnistModel.instance.removeTrainEventListener(this.onTrainingDone.bind(this))
    }
}
