# Script for converting MNIST dataset to .dat files.
#
# Usage:
#   python3 ubyte2dat.py train-images-idx3-ubyte train-labels-idx1-ubyte train.dat
#   python3 ubyte2dat.py t10k-images-idx3-ubyte t10k-labels-idx1-ubyte test.dat

import argparse
import array

def load_images(path):
    with open(path, "rb") as f:
        magic = int.from_bytes(f.read(4), "big")
        size = int.from_bytes(f.read(4), "big")
        rows = int.from_bytes(f.read(4), "big")
        columns = int.from_bytes(f.read(4), "big")

        if magic != 2051:
            print("Broken images file")
            exit(0)

        images = []
        for i in range(size):
            image = [b/255.0 for b in f.read(rows * columns)]
            images.append(image)

        return images


def load_labels(path):
    with open(path, "rb") as f:
        magic = int.from_bytes(f.read(4), "big")
        size = int.from_bytes(f.read(4), "big")

        if magic != 2049:
            print("Broken labels file")
            exit(0)

        return [b for b in f.read(size)]


def make_dataset_file(path, images, labels):
    if len(images) != len(labels):
        print("Images file and labels file do not match")
        exit(0)

    with open(path, "wb") as f:
        for x, y in zip(images, labels):
            array.array("f", x).tofile(f)
            z = [0.0] * 10
            z[y] = 1.0
            array.array("f", z).tofile(f)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("images", type=str)
    parser.add_argument("labels", type=str)
    parser.add_argument("output", type=str)
    args = parser.parse_args()

    images = load_images(args.images)
    labels = load_labels(args.labels)
    make_dataset_file(args.output, images, labels)


if __name__ == "__main__":
    main()

